import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import org.omg.CORBA.*;
import WebFlow.*;
import WebFlow.event.*;
import java.io.*;
import java.net.*;
import java.util.*;

//swing interface
import javax.swing.*;


public class ProcessTree extends JApplet implements ActionListener {

 private ORB orb;
 private String IOR;
 private WebFlowContext ws = null;
 //shows the current modules hierarchy 
 private JTextArea status;  
 //editfield for the root module IOR file URL
 private JTextField URLfield; 
 private String IorURL;
 //Offset to indicate step left for each next layer
 private String Offset = "    "; 

  //=============================functions declaration=========================

  public void init() {

     //create orb
     if (!createOrb()) {
        status.append("Error creating ORB");
        System.out.println("Error creating ORB");
        return;
     }
 
     IorURL = getParameter("IorURL");
     if (IorURL==null) IorURL = new String(); 
     System.err.println(IorURL);    

     //set up the applet interface
     setAppletInterface("");

     IOR = ReadSTRFromURL(IorURL);
     if (IORtoWebFlowContext(IOR)) {
       //  status.append("\n\nEstablished connection\n");
     } else {
        status.append("\n\nError establishing connection\n");
        return;
     }

     //Debugging purposes only, ot be removed
      if (ws.isWFServer()) {
        System.out.println(ws.getObjectID() + " is a server\n ");
      }  else {
        System.out.println(ws.getObjectID() +" is a Context\n ");
      }
      //end debugging code
     //Processing Hierarchy
     System.out.println("Tree for the master IOR file URL " + IorURL + "\n");
     status.append("Tree for the master IOR file URL " + IorURL +"\n"); 
     status.append(getHierarchy(0, ws));
  }
 
  //create ORB
  public boolean createOrb()
  {
    try {
      orb = ORB.init(this, new java.util.Properties()); 
      return true;
    }
    catch (Exception e) {
      orb=null;
      return false;
    }
  }
   
  //Accepts IOR of the server to connect
  //Returns true if connects false otherwise
  public boolean IORtoWebFlowContext(String IOR) {
    try {
      org.omg.CORBA.Object MasterObj=orb.string_to_object(IOR);
      if (MasterObj!=null)
	System.out.println("orb.string_to_object Ok");
      else {
	System.out.println("orb.string_to_object returned null");
	return false;
      }
      ws = WebFlowContextHelper.narrow(MasterObj);
      System.out.println("Narrowed");
      return true;
    }
    catch (Exception e) { 
      return false;
    } 
  }
  
  //recursively adds module hierarchy into the string result
  public String getHierarchy(int level, WebFlowContext node) {
 
    org.omg.CORBA.Object[] SlaveServers = node.toArray();

    //get the number of staces at the beginning of the line to pront "nice"
    String thisOffset = new String("");
    for (int j=0; j<level; j++)
      thisOffset+= Offset;

    String result = thisOffset + node.getObjectID();
    result += (node.isWFServer() ? " is a server ":" is a Context ");
    result += " with  "+ SlaveServers.length + " Child Objects\n";
  
    for (int i=0; i<SlaveServers.length; i++)
      {
	if (SlaveServers[i]._is_a("IDL:WebFlow/WebFlowContext:1.0")) {
	  try {
	    WebFlowContext isContext = WebFlowContextHelper.narrow(SlaveServers[i]);
	    System.out.println(result);
	    result += thisOffset +  getHierarchy(level+1, isContext);
	  } catch (Exception e) {
	    System.out.println("Could not narrow to WebFlowContext " + e);
	  } 
	}  
	else  {
	  try {
	    BeanContextChild Something = BeanContextChildHelper.narrow(SlaveServers[i]);       
	    result += thisOffset + Offset + Something.getObjectID() + " Is a module\n";
	    System.out.println(result);
	  } catch (Exception e) {
	    System.out.println("Could not narrow to BeanContextChild " + e);
	  }
	}
      }
    return result; 
  }  //getHierarchy
  

  //sets the applet's interface elements
  void setAppletInterface(String initLine) {
    //master layout
    getContentPane().setLayout(new BorderLayout());

    //create the panel with the processing button, editfield, and a label
    JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(new GridLayout(1,2));
    //button
    JButton getTreeButton = new JButton("Browse Process Tree from node URL:");
    getTreeButton.addActionListener(this);
    buttonPanel.add(getTreeButton);
    //editfield
    URLfield = new JTextField(IorURL);
    buttonPanel.add(URLfield);
    getContentPane().add(buttonPanel,BorderLayout.NORTH);
     
    //status area
    status = new JTextArea(initLine); 
    JScrollPane scrollPane = new JScrollPane(status,
                                   JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
                                   JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
    status.setEditable(false); 
    getContentPane().add(scrollPane,BorderLayout.CENTER);
  }
   
  //Action for the pressing button
  public void actionPerformed(ActionEvent ev) {
                        
    IorURL = URLfield.getText();
    System.out.println(IorURL);
    IOR = ReadSTRFromURL(IorURL);
    if (IORtoWebFlowContext(IOR)) {
      //status.append("\n\nEstablished connection\n");
    } else {
      status.append("\n\nError establishing connection\n");
      return;
    }
    //Processing Hierarchy
    System.out.println("\nTree for the master IOR file URL " + IorURL + "\n");
    status.append("\nTree for the master IOR file URL " + IorURL +"\n"); 
    status.append(getHierarchy(0, ws));           
  }

  //----------------reads object reference from a file------------
  public String ReadSTRFromFile(String FileName)   {
      //open the file
      FileInputStream theFile;
      BufferedReader fileReader;
      String IOR;

      try {
           theFile=new FileInputStream(FileName);
           fileReader=new BufferedReader(new InputStreamReader(theFile));
      }
      catch(FileNotFoundException e){
           return "";
      }

      //read a reference string
      try {
        IOR=fileReader.readLine();
        System.err.println("Got the reference: "+IOR);
      }
      catch (IOException e) {
        System.err.println("Exception while reading the file: " + e);
        return "";
      }
      
      try {
        theFile.close();
        fileReader.close();
      }
      catch (IOException e) { 
         System.err.println("Exception while closing file " + e);
         return "";
      }

      //return the reference string if Ok
      return IOR;
  }

  //reads text from servlet with the given URL
  //Does not separate lines
  public String ReadSTRFromURL(String sURL)
  {
    String fromServlet="";
    String LineIn="";

    try {
      java.net.URL url = new java.net.URL(sURL);
      InputStream istream;

      try {
        istream = url.openStream();
        BufferedReader in = new BufferedReader(new InputStreamReader(istream));
        
        while ( (LineIn=in.readLine()) !=null) 
           fromServlet +=LineIn;
	//String ref = in.readLine();
        istream.close();
        return fromServlet;
      }
      catch(Exception e) {
        System.err.println("Error " + e +" while reading from: " + sURL);
        return "Error " + e + " while reading from: " + sURL;
      }
    }  //outer try
    catch(Exception e) {
      System.err.println("Can't open URL: " + sURL);
      return "Can't open URL: " + sURL;
    }
  }  //function
  

}




